﻿using Hotline.Ai.Quality;
using Hotline.Quality;
using Hotline.Share.Dtos.Quality;
using Hotline.Share.Enums.Quality;
using MapsterMapper;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Quality
{
	public class QualityApplication : IQualityApplication, IScopeDependency
	{
		private readonly ISessionContext _sessionContext;
		private readonly IMapper _mapper;
		private readonly IRepository<QualityDetail> _qualiteyDetail;
		private readonly IQualityRepository _qualityRepository;
		private readonly IAiQualityService _aiQualityService;

		public QualityApplication(
			ISessionContext sessionContext,
			IMapper mapper,
			IRepository<QualityDetail> qualiteyDetail,
			IQualityRepository qualityRepository,
			IAiQualityService aiQualityService
		)
		{
			_sessionContext = sessionContext;
			_mapper = mapper;
			_qualiteyDetail = qualiteyDetail;
			_qualityRepository = qualityRepository;
			_aiQualityService = aiQualityService;
		}

		public async Task AddQualityAsync(EQualitySource Source, string OrderId, string VisitId, CancellationToken cancellationToken)
		{
			QualityAddDto quality = new QualityAddDto
			{
				Source = Source,
				OrderId = OrderId,
				VisitId = VisitId
			};
			var count = await _qualityRepository.CountAsync(x => x.OrderId == OrderId && x.Source == Source, cancellationToken);
			if (count <= 0)
				await AddQualityAsync(quality, cancellationToken);
		}

		public async Task AddQualityAsync(EQualitySource Source, string OrderId, CancellationToken cancellationToken)
		{
			QualityAddDto quality = new QualityAddDto
			{
				Source = Source,
				OrderId = OrderId
			};
			var count = await _qualityRepository.CountAsync(x => x.OrderId == OrderId && x.Source == Source, cancellationToken);
			if (count <= 0)
				await AddQualityAsync(quality, cancellationToken);
		}

		public async Task AddQualityAsync(QualityAddDto model, CancellationToken cancellationToken)
		{
			var quality = _mapper.Map<Hotline.Quality.Quality>(model);
			var id = await _qualityRepository.AddAsync(quality, cancellationToken);
			if (model.QualityDetails.Any())
			{
				foreach (var item in model.QualityDetails)
				{
					item.QualityId = id;
				}
				List<QualityDetail> details = _mapper.Map<List<QualityDetail>>(model.QualityDetails);
				await _qualiteyDetail.AddRangeAsync(details, cancellationToken);
			}
		}

		public async Task UpdateQualityAsync(UpdateQualityDto model, CancellationToken cancellationToken)
		{
			//验证工单是否可以申请
			var quality = await _qualityRepository.GetAsync(model.Id, cancellationToken);
			if (quality is null)
				throw UserFriendlyException.SameMessage("无效质检");
			_mapper.Map(model, quality);
			quality.State = EQualityState.End;
			quality.UserName = _sessionContext.UserName;
			quality.QualityTime = DateTime.Now;
			await _qualityRepository.UpdateAsync(quality, cancellationToken);
			List<QualityDetail> deleteDetails = await _qualiteyDetail.Queryable().Where(x => x.QualityId == model.Id).ToListAsync(cancellationToken);
			await _qualiteyDetail.RemoveRangeAsync(deleteDetails, cancellationToken);
			if (model.QualityDetails.Any())
			{
				foreach (var item in model.QualityDetails)
				{
					item.QualityId = model.Id;
				}
				List<QualityDetail> details = _mapper.Map<List<QualityDetail>>(model.QualityDetails);
				await _qualiteyDetail.AddRangeAsync(details, cancellationToken);
			}
		}
	}
}
